#!/usr/bin/perl
## ######### PROJECT NAME : ##########
## 
## Objet_manage.pl for Objet_manage.pl in /home/alex
## 
## ######### PROJECT DESCRIPTION : ###
## 
## L'oiseau est dans le
## le nid.
## 
## ###################################
## 
## Made by Boutonnet Alexandre
## Login   <alex@intellique.com>
## 
## Started on  Wed May 31 17:12:29 2006 Boutonnet Alexandre
## Last update Wed Sep 19 12:20:23 2007 Boutonnet Alexandre
## 
## ###################################
##

use strict;
use warnings;

package Objet_manage_disk;

use base 'Exporter';
our @EXPORT_OK = qw( new set_hardware_conf verif_conf set_conf get_conf );

use Objet_lang;
use lib_infodisks;

use Data::Dumper;

use Objet_disk;

# Constucteur : 

sub new
{
# ($event, [$entity], ["DEBUG"])
# - "DEBUG" est optionnel : s'il est present, on active le debug
# - $event est une reference sur l'objet event.
# - $entity est un parametre optionnel qui correspond  une string contenant le nom de l'objet fils que l'on doit
# instancier. Dans le cas o cette methode est appelle apres l'instanciation de l'objet pere, l'objet pere n'est pas
# recree. Cela sert pour forcer la creation d'un objet fils apres la detection du materiel (ex : interface reseau
# virtuelle). $entity doit contenir le nom de l'objet a creer exemple : "eth0:0".
# Cf Objet_storiq_install::create_entity() pour plus de dtail.
# Le constructeur instancie tous les objet_*.

    my ($OBJNAME, $EVENT, $entity, $isdebug) = @_;

    # L'option entity est geree au tout debut
    # pour eviter de tout ecraser...
    if ($entity and ($entity ne "DEBUG"))
    {
	# Je m'assure que tous les caracteres sont en Minuscule
	$entity =~ tr/[A-Z]/[a-z]/;

	# recuperation de la hash $OM
	my $OM = $OBJNAME;

	# je verifie que mon device n'existe deja pas
	if (exists($OM->{CURRENT}->{$entity}))
	{
	    # je pte une erreur ...
	    _event($OM, "ERROR", $OM->{LANG}->get_msg("error_exist_objet", $entity));
	    # ... et je quitte.
	    return (0);	    
	}

	# J'execute fdisk pour savoir si le device existe
	my @tab = qx(fdisk -l $entity 2> /dev/null);

	# si le tableau existe c'est que le device existe
	if (@tab)
	{
	    # instanciation du nouvel objet.
	    $OM->{CURRENT}->{$entity} = new Objet_disk($OM->{EVENT}, $entity, ($OM->{DEBUG} == 1) ? "DEBUG" : undef);
	    if (!$OM->{CURRENT}->{$entity})
	    {
		print STDERR "[ERROR] Objet_disk : unable to instanciate library\n";
		return 0;
	    }
	    $OM->{CORRESPONDANCE}->{$OM->{CURRENT_NUMBER}++} = $entity;
	}
	else
	{
	    # je pte une erreur ...
	    _event($OM, "ERROR", $OM->{LANG}->get_msg("error_notexist_device", $entity));
	    # ... et je quitte.
	    return (0);
	}

	return (1);
    }

    my $OM = {};

    $OM->{OBJNAME} = $OBJNAME;
    $OM->{EVENT} = $EVENT;
    $OM->{CURRENT} = {};

    # cette hash me permet de faire la correspondance disk0 <=> /dev/hda..
    # dedans je stoque que 0, 1, 2.. etc
    $OM->{CORRESPONDANCE} = {};

    # ce int est ne nombre de disque que je gre
    $OM->{CURRENT_NUMBER} = 0;

    # Si l'objet event est absent, je stop l'instanciation et je retourne le code d'erreur.
    if (!$OM->{EVENT})
    {
	print STDERR "[ERROR] $OM->{OBJNAME} : new : Le parametre $OM->{EVENT} est absent\n";
	return;
    }
    unless (ref($OM->{EVENT}) eq "Objet_Event")
    {
	print STDERR "[ERROR] $OM->{OBJNAME} : new : Le parametre $OM->{EVENT} n'est pas un Objet_Event\n";
	return;
    }

    ### gestion du debug

    # je gre le cas pourri si $entity est absent
    if (!$isdebug and $entity and ($entity eq "DEBUG"))
    {
	$isdebug = $entity;
    }

    # Je gere le debug ..
    $OM->{DEBUG} = 0;
    if ($isdebug and ($isdebug eq "DEBUG"))
    {
	$OM->{DEBUG} = 1;
    }

    # Hop je recupere le tableau qui contient tout les disques.
    my @tab_disk = lib_infodisks::get_disk_list();

    # je boucle dessus pour instancier tous mes objets fils
    foreach my $dd (@tab_disk)
    {
	chomp($dd);
	$OM->{CURRENT}->{$dd} = new Objet_disk($OM->{EVENT}, $dd, ($OM->{DEBUG} == 1) ? "DEBUG" : undef);
	if (!$OM->{CURRENT}->{$dd})
	{
	    print STDERR "[ERROR] Objet_disk : unable to instanciate library\n";
	    return 0;
	}
	
	$OM->{CORRESPONDANCE}->{$OM->{CURRENT_NUMBER}++} = $dd;
    }
 
    # initialisation de l'objet de langue
    $OM->{LANG} = Objet_lang->new($OM->{EVENT}, "Objet_manage", ($OM->{DEBUG} == 1) ? "DEBUG" : undef);
    if (!$OM->{LANG})
    {
	print STDERR "[ERROR] Objet_lang : unable to instanciate library\n";
	return 0;
    }

    bless ($OM);
    return ($OM);
}


# Initialisation du module : Cette methode lance la detection du materiel et le sauvegarde.

sub set_hardware_conf
{
# ([$identity])
# - $identity est un parametre optionnel. Si l'on passe "disk0", par exemple, la fonction s'appliquera uniquement
# a l'objet_* correspondant.
# - Cette methode retourne un bool : true en cas de succes et false si quelque chose a echoue. Des infos/warning/erreurs
# sont places dans l'objet event.
# Cette methode appelle la methode set_hardware_conf(void) du/des objet_* fils.

    my ($OM, $entity) = @_;

    # Je suppose que l'exec de cette methode va bien se terminer..
    my $flag_return = 1;
    
    # si entity exist
    if ($entity)
    {

	# je recupere le numero qui suit "disk"
	my $number = ($entity =~ m/disk(\d+)/)[0];

	# Je teste que j'ai bien match un nombre..
	if (!defined($number))
	{
	    _event($OM, "ERROR", $OM->{LANG}->get_msg("error_notexist_objet", $entity));
	    return (0);
	}

	# Je check s'il exist une correspondance...
	if (!exists($OM->{CORRESPONDANCE}->{$number}))
	{
	    # S'il y en a pas, je pte un warning
	    _event($OM, "WARN", $OM->{LANG}->get_msg("error_notexist_objet", $entity));
	    $flag_return = 0;
	}
	# sinon j'appelle verif_conf du bon fils
	elsif (!$OM->{CURRENT}->{$OM->{CORRESPONDANCE}->{$number}}->set_hardware_conf())
	{
	    $flag_return = 0;
	}

##### Hum je sais plus o j'en suis l...
# 	# je recherche l'objet qui correspond.
# 	if (exists($OM->{CURRENT}->{$entity}))
# 	{
# 	    # hop j'appelle mon fils
# 	    $flag_return = $OM->{CURRENT}->{$entity}->set_hardware_conf();
# 	}
# 	# si cet objet n'existe pas, je pte une erreur
# 	else
# 	{
# 	    # je pte une erreur ...
# 	    _event($OM, "ERROR", $OM->{LANG}->get_msg("error_notexist_objet", $entity));
# 	    # ... et je quitte.
# 	    $flag_return = 0;
#  	}
    }
    else
    {
	# Je boucle sur tous mes fils..
	foreach my $Objet (keys(%{$OM->{CURRENT}}))
	{
	    # et je les appelle chacun leur tour :)
	    if (!$OM->{CURRENT}->{$Objet}->set_hardware_conf())
	    {
		$flag_return = 0;
	    }
	}
    }

    return ($flag_return);
}


# Verification de la configuration : Cette fonction verifie la configuration que l'on lui passe en parametre

sub verif_conf
{
# (\%conf)
# - %conf est une reference sur une hash de hash de tableau. Elle contient la/les configuration(s) a tester.
# Exemple : Dans le cas de l'objet Objet_manage_disk, %conf pourra ressembler a cela :
#  disk0 =>  device =>     [/dev/hda, $error_msg]
#            label =>      [gpt, $error_msg]
#            size =>       [214748364800, $error_msg]
#            model_name => [MaXLine Pro 500, $error_msg]
#  disk1 =>  device =>     [/dev/sda, $error_msg]
#            label =>      [gpt, $error_msg]
#            size =>       [214748364800, $error_msg]
#            model_name => [MaXLine Pro 500, $error_msg]
# $error_msg contient le message d'erreur qui est genere par le fils en cas d'echec sur cette partie de configuration.
# Cette methode retourne un bool : true si l'operation a reussi et false en cas d'erreur.

    my ($OM, $conf) = @_;

    # je set le flag a 1 : je suppose que la methode va correctement se terminer.
    my $flag_error = 1;

    # je teste si le parametre conf est pas absent
    unless (defined($conf))
    {
	_event($OM, "ERROR", $OM->{LANG}->get_msg("error_verif_conf_param", "Objet_manage_services", "\$conf"));
	return (0);
    }

    # je boucle sur les differentes conf de la hash
    foreach my $disk (keys(%{$conf}))
    {
	# je recupre le numro qui suit "disk"
	my $number = ($disk =~ m/disk(\d+)/)[0];

	# Je teste que j'ai bien match un nombre..
	if (!defined($number))
	{
	    _event($OM, "ERROR", $OM->{LANG}->get_msg("error_notexist_objet", $disk));
	    return (0);
	}

	# Je check s'il exist une correspondance...
	if (!exists($OM->{CORRESPONDANCE}->{$number}))
	{
	    # S'il y en a pas, je pte un warning
	    _event($OM, "WARN", $OM->{LANG}->get_msg("error_notexist_objet", $disk));
	    $flag_error = 0;
	}
	# sinon j'appelle verif_conf du bon fils
	elsif (!$OM->{CURRENT}->{$OM->{CORRESPONDANCE}->{$number}}->verif_conf($conf->{$disk}))
	{
	    $flag_error = 0;
	}
    }

    # coin coin
    return ($flag_error);
}


# Applique une configuration  un objet : Cette configuration est testee avant d'etre appliquee

sub set_conf
{
# (\%conf)
# - %conf est une reference sur une hash de hash de tableau. Elle contient la/les configuration(s) a appliquer.
# Exemple : Dans le cas de l'objet Objet_manage_disk, %conf pourra ressembler a cela :
#  disk0 =>  device =>     [/dev/hda, $error_msg]
#            label =>      [gpt, $error_msg]
#            size =>       [214748364800, $error_msg]
#            model_name => [MaXLine Pro 500, $error_msg]
#  disk1 =>  device =>     [/dev/sda, $error_msg]
#            label =>      [gpt, $error_msg]
#            size =>       [214748364800, $error_msg]
#            model_name => [MaXLine Pro 500, $error_msg]
# $error_msg contient le message d'erreur qui est genere par le fils en cas d'echec sur cette partie de configuration.
# Cette methode retourne un bool : true si l'operation a reussi et false en cas d'erreur.

    my ($OM, $conf) = @_;

    # je set le flag a 1 : je suppose que la methode va correctement se terminer.
    my $flag_error = 1;

    # je teste si le parametre conf est pas absent
    unless (defined($conf))
    {
	_event($OM, "ERROR", $OM->{LANG}->get_msg("error_verif_conf_param", "Objet_manage_services", "\$conf"));
	return (0);
    }

    # je boucle sur les differentes conf de la hash
    foreach my $disk (keys(%{$conf}))
    {
	# je recupre le numro qui suit "disk"
	my $number = ($disk =~ m/disk(\d+)/)[0];

	# Je teste que j'ai bien match un nombre..
	if (!defined($number))
	{
	    _event($OM, "ERROR", $OM->{LANG}->get_msg("error_notexist_objet", $disk));
	    return (0);
	}

	# Je check s'il exist une correspondance...
	if (!exists($OM->{CORRESPONDANCE}->{$number}))
	{
	    # S'il y en a pas, je pte un warning
	    _event($OM, "WARN", $OM->{LANG}->get_msg("error_notexist_objet", $disk));
	    $flag_error = 0;
	}
	# sinon j'appelle verif_conf du bon fils
	elsif (!$OM->{CURRENT}->{$OM->{CORRESPONDANCE}->{$number}}->set_conf($conf->{$disk}))
	{
	    $flag_error = 0;
	}
    }

    # coin coin
    return ($flag_error);
}


# Recuperation de la configuration 

sub get_conf
{
# ([$identity])
# - $identity est optionnel. si $identity est present il doit contenir l'objet a recuperer (exemple : disk0). Si $identity
# est invalide, get_conf retourne undef. Si $identify est abscent ou nul, get_conf retourne la configuration de tous
# les objets fils.
# Cette fonction retourne la configuration. Concraitement c'est un reference sur une une hash.
# Exemple : Dans le cas de l'objet Objet_manage_disk, %conf pourra ressembler a cela :
#  disk0 =>  device =>     [/dev/hda, $error_msg]
#            label =>      [gpt, $error_msg]
#            size =>       [214748364800, $error_msg]
#            model_name => [MaXLine Pro 500, $error_msg]
#  disk1 =>  device =>     [/dev/sda, $error_msg]
#            label =>      [gpt, $error_msg]
#            size =>       [214748364800, $error_msg]
#            model_name => [MaXLine Pro 500, $error_msg]

    my ($OM, $entity) = @_;

    # J'init la hash de retour
    my $hash = {};
    
    # si entity exist
    if ($entity)
    {

	# je recupere le numero qui suit "disk"
	my $number = ($entity =~ m/disk(\d+)/)[0];

	# Je teste que j'ai bien match un nombre..
	if (!defined($number))
	{
	    _event($OM, "ERROR", $OM->{LANG}->get_msg("error_notexist_objet", $entity));
	    return (0);
	}

	# Je check s'il exist une correspondance...
	if (!exists($OM->{CORRESPONDANCE}->{$number}))
	{
	    # S'il y en a pas, je pte un warning
	    _event($OM, "ERROR", $OM->{LANG}->get_msg("error_notexist_objet", $entity));
	    return (0);
	}
	$hash->{$entity} = $OM->{CURRENT}->{$OM->{CORRESPONDANCE}->{$number}}->get_conf();

######## j'ai plus ma tete :'(

# 	# je recherche l'objet qui correspond.
# 	if (exists($OM->{CURRENT}->{$entity}))
# 	{
# 	    # je recupere le chiffre qui correspond
# 	    my $Num;
# 	    foreach my $Num_tmp (keys(%{$OM->{CORRESPONDANCE}}))
# 	    {
# 		if ($OM->{CORRESPONDANCE}->{$Num_tmp} eq $entity)
# 		{
# 		    $Num = $Num_tmp;
# 		    last;
# 		}
# 	    }
# 	    # hop j'appelle mon fils
# 	    $hash->{"disk$Num"} = $OM->{CURRENT}->{$entity}->get_conf();
# 	}
# 	# si cet objet n'existe pas, je pte une erreur
# 	else
# 	{
# 	    # je pte une erreur ...
# 	    _event($OM, "ERROR", $OM->{LANG}->get_msg("error_notexist_objet", $entity));
# 	    # ... et je quitte.
#  	}
    }
    else
    {
	# Je boucle sur tous mes fils en passant par les correspondance..
	foreach my $Num (keys(%{$OM->{CORRESPONDANCE}}))
	{
	    # et je les appelle chacun leur tour :)
	    $hash->{"disk".$Num} = $OM->{CURRENT}->{$OM->{CORRESPONDANCE}->{$Num}}->get_conf();
	}
    }

    return ($hash);
}

#########################################################################################
####            METHODES PRIVEES
#########################################################################################

sub _event
{
# _event($self, $type, $msg);
# Gestion des evenements l'objet conf

    my ($self, $type, $msg) = @_;

    return if ($type eq "DEBUG" and !$self->{DEBUG});
    return $self->{EVENT}->set_event($type, time(),$self->{OBJNAME} . " : " . $msg);
}

1;

###################################################################################################

=head1 NOM

Objet Manage Disk - Objet de verification de la configuration materielle des disques durs.

=head1 DEPENDANCE

Cet Objet depend de l'objet event (Objet_event) ainsi que des objets ci dessous :
 - Objet_disk
 - Objet_lang

=head1 PROTOTYPAGE

B<new>($event, [$entity], ["DEBUG"],);                              [1 ou undef]

B<set_hardware_conf>([$identity]);                                 [true ou false]

B<verif_conf>(\%conf);                                             [true ou false]

B<set_conf>(\%conf);                                               [true ou false]

B<get_conf>([$identity]);                                          [\%conf]

=head1 DEFINITION

B<new> : Constructeur de l'objet. "DEBUG" est optionnel : s'il est present, on active le debug. $event est une reference sur l'objet event. $entity est un parametre optionnel qui correspond  une string contenant le nom de l'objet fils que l'on doit instancier. Dans le cas o cette methode est appelle apres l'instanciation de l'objet pere, l'objet pere n'est pas recree. Cela sert pour forcer la creation d'un objet fils apres la detection du materiel (ex : interface reseau virtuelle). $entity doit contenir le nom de l'objet a creer exemple : "/dev/hda".
 Cf Objet_storiq_install::create_entity() pour plus de dtail. Le constructeur instancie tous les objet_*.

B<set_hardware_conf> : Initialisation du module : Cette methode lance la detection du materiel et le sauvegarde. $identity est un parametre optionnel. Si l'on passe "disk0", par exemple, la fonction s'appliquera uniquement a l'objet_* correspondant.
 Cette methode retourne un bool : true en cas de succes et false si quelque chose a echoue. Des infos/warning/erreurs sont places dans l'objet event.
 Cette methode appelle la methode set_hardware_conf(void) du/des objet_* fils.

B<verif_conf> : Verification de la configuration : Cette fonction verifie la configuration que l'on lui passe en parametre. %conf est une reference sur une hash de hash de tableau. Elle contient la/les configuration(s) a tester.

Exemple : Dans le cas de l'objet Objet_manage_disk, %conf pourra ressembler a cela :
 disk0 =>  device =>     [/dev/hda, $error_msg]
           label =>      [gpt, $error_msg]
           size =>       [30000000, $error_msg]
           model_name => [MaXLine Pro 500, $error_msg]
 disk1 =>  device =>     [/dev/sda, $error_msg]
           label =>      [gpt, $error_msg]
           size =>       [214748364800, $error_msg]
           model_name => [MaXLine Pro 500, $error_msg]

$error_msg contient le message d'erreur qui est genere par le fils en cas d'echec sur cette partie de configuration.
 Cette methode retourne un bool : true si l'operation a reussi et false en cas d'erreur.

B<set_conf> Applique une configuration  un objet : Cette configuration est testee avant d'etre appliquee. %conf est une reference sur une hash de hash de tableau. Elle contient la/les configuration(s) a appliquer.

Exemple : Dans le cas de l'objet Objet_manage_disk, %conf est du meme type que dans la methode verif_conf. Je vous invite a vous reporter a cet exemple.

$error_msg contient le message d'erreur qui est genere par le fils en cas d'echec sur cette partie de configuration.
 Cette methode retourne un bool : true si l'operation a reussi et false en cas d'erreur.

B<get_conf> : Recuperation de la configuration. $identity est optionnel. Si $identity est present il doit contenir l'objet a recuperer (exemple : disk0). Si $identity est invalide, get_conf retourne undef. Si $identify est abscent ou nul, get_conf retourne la configuration de tous les objets fils. Cette fonction retourne la configuration. Concraitement c'est un reference sur une une hash.

Exemple : Dans le cas de l'objet Objet_manage_disk, %conf pourra ressembler a cela :
 disk0 =>  device =>     [/dev/hda, $error_msg]
           label =>      [gpt, $error_msg]
           size =>       [30000000 Go, $error_msg]
           vendor =>     [Maxtor, $error_msg]
           model_name => [MaXLine Pro 500, $error_msg]
 disk1 =>  device =>     [/dev/sda, $error_msg]
           label =>      [gpt, $error_msg]
           size =>       [214748364800 Go, $error_msg]
           vendor =>     [Maxtor, $error_msg]
           model_name => [MaXLine Pro 500, $error_msg]

=head1 UTILISATION

#Utilisation de l'objet manage disk :
    use Objet_manage_disk;
    use Objet_Event;

#Instanciation de l'objet manage disk :
    my $ObjetManage = new Objet_manage_disk($event);

#Initialisation du module. Cette methode lance la detection du materiel et le sauvegarde :
    $ObjetManage->set_hardware_conf();
    #ou
    $ObjetManage->set_hardware_conf("/dev/hda");

#Verification de la configuration :
    my %conf;
    $ObjetManage->verif_conf(\%conf);

#Applique une configuration  un objet :
    $ObjetManage->set_conf(\%conf);

#Recuperation de la configuration :
    $ObjetManage->get_conf();
    #ou
    $ObjetManage->get_conf("disk0");


=head1 AUTEUR

Boutonnet Alexandre <aboutonnet [at] intellique.com>

