#!/usr/bin/perl
## ######### PROJECT NAME : ##########
## 
## Objet.pl for Objet.pl in /home/alex
## 
## ######### PROJECT DESCRIPTION : ###
## 
## L'oiseau est dans le
## le nid.
## 
## ###################################
## 
## Made by Boutonnet Alexandre
## Login   <alex@intellique.com>
## 
## Started on  Wed May 31 17:16:28 2006 Boutonnet Alexandre
## Last update Wed Sep 19 11:38:14 2007 Boutonnet Alexandre
## 
## ###################################
##

package Objet_partition;

use strict;
use warnings;

use Objet_lang;
use lib_infodisks;

use base 'Exporter';
our @EXPORT_OK = qw( new set_hardware_conf verif_conf set_conf get_conf );

use Data::Dumper;
# Constucteur : 

sub new
{
# ($event, $device, ["DEBUG"])
# - $device est le nom du device a gerer..
# - "DEBUG" est optionnel : s'il est present, on active le debug
# - $event est une reference sur l'objet event.

    my ($OBJNAME, $EVENT, $device, $isdebug,) = @_;

    my $OBJ = {};

    $OBJ->{OBJNAME} = $OBJNAME;
    $OBJ->{EVENT} = $EVENT;
    $OBJ->{CURRENT} = {};

    # Si l'objet event est absent, je stop l'instanciation et je retourne le code d'erreur.
    if (!$OBJ->{EVENT})
    {
	print STDERR "[ERROR] $OBJ->{OBJNAME} : new : \$OBJ->{EVENT} parameter is missing\n";
	return;
    }
    unless (ref($OBJ->{EVENT}) eq "Objet_Event")
    {
	print STDERR "[ERROR] $OBJ->{OBJNAME} : new : \$OBJ->{EVENT} parameter isn't an Objet_Event Object\n";
	return;
    }

    ### gestion du $device..
    # Check si le device est present
    if (!$device)
    {
	print STDERR "[ERROR] $OBJ->{OBJNAME} : new : \$device parametre is missing\n";
	return(0);
    }

    # Je massure que le device est en minuscule
    $device =~ tr/[A-Z]/[a-z]/;

    # Check si le nom de device est pas foireux
    if ($device !~ m/\/dev\/(?:sd|hd)[a-z]{1}\d+$/)
    {
	print STDERR "[ERROR] $OBJ->{OBJNAME} : new : \$device parametre is incorrect\n";
	return(0);
    }
    # Check si le device existe :
    # Nan, pas question.. sinan on peut pas creer de partoch..

    # Si tous ces tests sont passe c que $device est correct
    $OBJ->{DEVICE} = $device;

    ### gestion du debug
    # Je gere le debug ..
    $OBJ->{DEBUG} = 0;
    if ($isdebug and ($isdebug eq "DEBUG"))
    {
	$OBJ->{DEBUG} = 1;
    }

    # initialisation de l'objet de langue
    $OBJ->{LANG} = Objet_lang->new($OBJ->{EVENT}, "Objet_", ($OBJ->{DEBUG} == 1) ? "DEBUG" : undef);
    if (!$OBJ->{LANG})
    {
	print STDERR "[ERROR] Objet_lang : unable to instanciate library\n";
	return 0;
    }

    bless ($OBJ);
    return ($OBJ);
}


# Detection de la configuration materielle : Cette fonction detecte la configuration et l'applique  l'objet

sub set_hardware_conf
{
# (void)
# L'objet_* contient une methode privee qui permet de detecter la configuration materielle du "device" gere par ce dernier.
# Apres l'appel de cette methode privee, la configuration ainsi detecter est "enregistree" dans l'objet. Set_hardware_conf
# est chargee d'orchestrer le tout.

    my $OBJ = shift;

    # J'appelle la lib de recuperation des infos de partition.
    my $hash_info = lib_infodisks::get_partition_info($OBJ->{DEVICE});

    if (!$hash_info)
    {
	# Je retourne un warning..
	_event($OBJ, "WARN", $OBJ->{LANG}->get_msg("error_device_not_found",
						   "Objet_partition", "set_hardware_conf", "$OBJ->{DEVICE}"));
	$OBJ->{CURRENT}->{"size"} = ["-1", 0];
	$OBJ->{CURRENT}->{"file_system"} = ["unknown", 0];
    }

    # Je l'enregistre dans ma hash
    $OBJ->{CURRENT}->{"size"} = [$hash_info->{$OBJ->{DEVICE}}->{"size"}, 0] if ($hash_info);

    # J'enregistre mon fs :
    $OBJ->{CURRENT}->{"file_system"} = [$hash_info->{$OBJ->{DEVICE}}->{"filesystem"}, 0] if ($hash_info);

    # J'enregistre mon device
    $OBJ->{CURRENT}->{"name"} = [$OBJ->{DEVICE}, 0];

    return (1);
}


# Verification de la configuration : Cette fonction verifie la configuration que l'on lui passe en parametre

sub verif_conf
{
# (\%conf)
# - %conf est une reference sur une hash de tableau. Elle contient la configuration a tester.
# Exemple : Dans le cas de l'objet Objet_manage_network, %conf pourra ressembler a cela :
#  name => [/dev/sda2, $error_msg]
#  size => [1000, $error_msg]
#  file_system => [swap, $error_msg]
# - $error_msg contient le message d'erreur qui est genere en cas d'echec sur cette partie de configuration.
# Cette methode retourne un bool : true si l'operation a reussi et false en cas d'erreur.

    my ($OBJ, $conf) = @_;

    # je set le flag a 1 : je suppose que la methode va correctement se terminer.
    my $flag_error = 1;

    # je teste si le parametre conf est pas absent
    unless (defined($conf))
    {
	_event($OBJ, "ERROR", $OBJ->{LANG}->get_msg("error_verif_conf_param", "Objet_disk", "\$conf"));
	return (0);
    }

    # je verifie que "device" existe
    if (!exists($conf->{"device"}))
    {
	_event($OBJ, "ERROR", $OBJ->{LANG}->get_msg("hash_key_not_found", "verif_conf",
						    "device"));
	$flag_error = 0;
    }
    # Je teste si le "device" de la hash est le meme que pour mon objet
    elsif ($conf->{"device"}[0] ne $OBJ->{DEVICE})
    {
	$conf->{"device"}[1] = $OBJ->{LANG}->get_msg("verif_conf_device");
	$flag_error = 0;
    }

    # je verifie que "size" existe
    if (!exists($conf->{"size"}))
    {
	_event($OBJ, "ERROR", $OBJ->{LANG}->get_msg("hash_key_not_found", "verif_conf",
						    "size"));
	$flag_error = 0;
    }
    # Je verifie si la size est po pourrie
    elsif ($conf->{"size"}[0] !~ m/^\d+$/)
    {
	$conf->{"size"}[1] = $OBJ->{LANG}->get_msg("OP_verif_conf_size");
	$flag_error = 0;
    }


    # je verifie que "file_system" existe
    if (!exists($conf->{"file_system"}))
    {
	_event($OBJ, "ERROR", $OBJ->{LANG}->get_msg("hash_key_not_found", "verif_conf",
						    "file_system"));
	$flag_error = 0;
    }
    # Je verifie si le fs est pas errone
    elsif ($conf->{"file_system"}[0] ne "reiserfs"
	   && $conf->{"file_system"}[0] ne "ext2"
	   && $conf->{"file_system"}[0] ne "ext3"
	   && $conf->{"file_system"}[0] ne "xfs"
	   && $conf->{"file_system"}[0] ne "linux-swap"
#	&& $conf->{"file_system"}[0] ne "jfs" # Commente car pas encore supporte
	   && $conf->{"file_system"}[0] ne "vfat")
    {
	$conf->{"file_system"}[1] = $OBJ->{LANG}->get_msg("OP_verif_conf_fs");
	$flag_error = 0;
    }
    
    return ($flag_error);
}


# Applique une configuration  un objet : Cette configuration est testee avant d\'etre appliquee

sub set_conf
{
# (\%conf)
# - %conf est une reference sur une hash de tableau. Elle contient la configuration  appliquer.
# Exemple : Dans le cas de l'objet Objet_manage_network, %conf pourra ressembler a cela :
#  name => [/dev/sda2, $error_msg]
#  size => [1000, $error_msg]
#  file_system => [swap, $error_msg]
# - $error_msg contient le message d'erreur qui est genere en cas d'echec sur cette partie de configuration.
# Cette methode retourne un bool : true si l'operation a reussi et false en cas d'erreur.

    my ($OBJ, $conf) = @_;

    # je set le flag a 1 : je suppose que la methode va correctement se terminer.
    my $flag_error = 1;

    # je teste si le parametre conf est pas absent
    unless (defined($conf))
    {
	_event($OBJ, "ERROR", $OBJ->{LANG}->get_msg("error_set_conf_param", "Objet_partition", "\$conf"));
	return (0);
    }

    # je test si la conf est correcte avant de l'enregistrer
    if (!verif_conf($OBJ, $conf))
    {
	return (0);
    }

    # je copie ma hash..
    my %tmp = %{$conf};
    $OBJ->{CURRENT} = \%tmp;
    
    return ($flag_error);
}

# Recuperation de la configuration 

sub get_conf
{
# (void)
# Cette fonction retourne la configuration. Concretement c'est un reference sur une hash.
# Exemple : Dans le cas de l'objet Objet_manage_network, %conf pourra ressembler a cela :
#  name => [/dev/sda2, $error_msg]
#  size => [1000, $error_msg]
#  file_system => [swap, $error_msg]

    my $OBJ = shift;

    # je copie ma hash... dans une hash temporaire
    my %tmp = %{$OBJ->{CURRENT}};

    # Et pour je retourne une reference sur ma derniere hash
    return (\%tmp);
}

#########################################################################################
####            METHODES PRIVEES
#########################################################################################

sub _event
{
# _event($self, $type, $msg);
# Gestion des evenements l'objet conf

    my ($self, $type, $msg) = @_;

    return if ($type eq "DEBUG" and !$self->{DEBUG});
    return $self->{EVENT}->set_event($type, time(),$self->{OBJNAME} . " : " . $msg);
}

1;

###################################################################################################

=head1 NOM

Objet Partition - Objet de verification de la configuration materielle d'une partition.

=head1 DEPENDANCE

Cet Objet depend de l'objet event (Objet_event) et de l'objet langue (Objet_lang).

=head1 PROTOTYPAGE

B<new>($event, $device, ["DEBUG"]);                     [1 ou undef]

B<set_hardware_conf>(void);                             [true ou false]

B<verif_conf>(\%conf);                                  [true ou false]

B<set_conf>(\%conf);                                    [true ou false]

B<get_conf>(void);                                      [\%conf]

=head1 DEFINITION

B<new> : Constructeur de l'objet. "DEBUG" est optionnel : s'il est present, on active le debug. $event est une reference sur l'objet event. $device est le nom du device que l'objet est charge de gerer (ex: /dev/sda1).

B<set_hardware_conf> : Detection de la configuration materielle : Cette fonction detecte la configuration et l'applique  l'objet. L'objet contient une methode privee qui permet de detecter la configuration materielle du "device" gere par ce dernier. Apres l'appel de cette methode privee, la configuration ainsi detecter est "enregistree" dans l'objet. Set_hardware_conf est chargee d'orchestrer le tout.
 Cette methode retourne un bool : true en cas de succes et false si quelque chose a echoue. Des infos/warning/erreurs sont places dans l'objet event.

B<verif_conf> : Verification de la configuration : Cette fonction verifie la configuration que l'on lui passe en parametre. %conf est une reference sur une hash de tableau. Elle contient la configuration a tester.

Exemple : Dans le cas de l'objet Objet_partition, %conf pourra ressembler a cela :
 name => [/dev/sda2, $error_msg]
 size => [1000, $error_msg]
 file_system => [swap, $error_msg]

 $error_msg contient le message d'erreur qui est genere en cas d'echec sur cette partie de configuration.
 Cette methode retourne un bool : true si l'operation a reussi et false en cas d'erreur.

B<set_conf> : Applique une configuration  un objet : Cette configuration est testee avant d'etre appliquee. %conf est une reference sur une hash de tableau. Elle contient la configuration  appliquer.

Exemple : Dans le cas de l'objet Objet_partition, %conf pourra ressembler a cela :
 name => [/dev/sda2, $error_msg]
 size => [1000, $error_msg]
 file_system => [swap, $error_msg]

 $error_msg contient le message d'erreur qui est genere en cas d'echec sur cette partie de configuration.
 Cette methode retourne un bool : true si l'operation a reussi et false en cas d'erreur.

B<get_conf> : Recuperation de la configuration. Cette fonction retourne la configuration. Concretement c'est un reference sur une hash.

Exemple : Dans le cas de l'objet Objet_partition, %conf pourra ressembler a cela :
 name => /dev/sda2
 size => 1000
 file_system => swap

=head1 UTILISATION

#Utilisation de l'objet partition :
    use Objet_partition;
    use Objet_Event;

#Instanciation de l'objet partition :
    my $Objet = new Objet_partition($event, "/dev/sda2");

#Initialisation du module. Cette methode lance la detection du materiel et le sauvegarde :
    $Objet->set_hardware_conf();

#Verification de la configuration :
    my %conf;
    $Objet->verif_conf(\%conf);

#Applique une configuration  un objet :
    $Objet->set_conf(\%conf);

#Recuperation de la configuration :
    $Objet->get_conf();


=head1 AUTEUR

Boutonnet Alexandre <aboutonnet [at] intellique.com>

